;==============================================================================
; Generate the keywords.htm or functions.htm reference page from
; the specially formatted TXT files
;
; Input and output directories are read from 'txt2htm.ini'
; The directory should contain either keywords or functions; not both!
;
; 30 Jan 2004 - CyberSlug - philipgump@hotmail.com
; 18 may 2004 - JdeB - jdeb@autoitscript.com
;             - Added support for UDF files.
;             - Changed the way the link to the html file is made
; 18 Sep 2005 - jpm - jpm@autoitscript.com
;             - Added /AutoIt /UDFs command parameters
; 09 Oct 2007 - jpm - jpm@autoitscript.com
;             - Ignored $Tag... files
; 12 Mar 2013 - jpm - jpm@autoitscript.com
;             - Reorg to have same source for AutoItX generation
; Updated:  13 jul 2013 jos
;           - Changed the path for include to support building from DOCS directory
;==============================================================================

#include "OutputLib.au3"

#include <Constants.au3>
#include <File.au3>

Opt("TrayIconDebug", 1)

OnAutoItExitRegister("OnQuit") ; ### When run in standalone
_OutputWindowCreate() ; ### When run in standalone

Global $g_iReGen_AutoIt, $g_iReGen_UDFs, $g_iReGen_AutoItX = 0
$g_iReGen_AutoIt = StringInStr($CmdLineRaw, "/AutoIt")
$g_iReGen_UDFs = StringInStr($CmdLineRaw, "/UDFs")

If FileExists(@WorkingDir & "\AutoItX.hhp") Then
	$g_iReGen_AutoItX = 1
Else
	If $g_iReGen_AutoIt = 0 And $g_iReGen_UDFs = 0 Then
		; reGenerate AutoIt and UDFs
		$g_iReGen_AutoIt = 1
		$g_iReGen_UDFs = 1
	EndIf
EndIf

Global Const $TXT2HTM_INI = @WorkingDir & "\txt2htm.ini"
Global $g_sRefType ;is either "Function" or "Keyword"
Global $g_hOut ;handle to the output file
Global $g_hIn ;handle to current input file
Global $g_sINPUT_DIR, $g_sHtmlDir
Global $g_sOUTPUT_DIR = IniRead($TXT2HTM_INI, "Output", "mainHtml", "ERR")
Global $g_sRefTypeNotes ; Func Notes.

If $g_iReGen_AutoIt Then
	$g_sRefType = "Function"
	$g_sRefTypeNotes = "function_notes"
	$g_hOut = FileOpen($g_sOUTPUT_DIR & "functions.htm", BitOR($FO_OVERWRITE, $FO_UTF8)) ;overwrite mode
	$g_sINPUT_DIR = IniRead($TXT2HTM_INI, "Input", "functions", "ERR")
	; ### Added this to determine the link to the htm file
	$g_sHtmlDir = IniRead($TXT2HTM_INI, "Output", "functions", "ERR")
	genFile()
	;
	$g_sRefType = "Keyword"
	$g_sRefTypeNotes = ""
	$g_hOut = FileOpen($g_sOUTPUT_DIR & "keywords.htm", BitOR($FO_OVERWRITE, $FO_UTF8)) ;overwrite mode
	$g_sINPUT_DIR = IniRead($TXT2HTM_INI, "Input", "keywords", "ERR")
	; ### Added this to determine the link to the htm file
	$g_sHtmlDir = IniRead($TXT2HTM_INI, "Output", "keywords", "ERR")
	genFile()
EndIf

If $g_iReGen_UDFs Then
	; ### Added this section for UDFS
	;
	$g_sRefType = "User Defined Function"
	$g_sRefTypeNotes = "libfunction_notes"
	$g_hOut = FileOpen($g_sOUTPUT_DIR & "libfunctions.htm", BitOR($FO_OVERWRITE, $FO_UTF8)) ;overwrite mode
	$g_sINPUT_DIR = IniRead($TXT2HTM_INI, "Input", "libfunctions", "ERR")
	; ### Added this to determine the link to the htm file
	$g_sHtmlDir = IniRead($TXT2HTM_INI, "Output", "libfunctions", "ERR")
	genFile()

	GenSubRefs()

EndIf

If $g_iReGen_AutoItX Then
	$g_sRefType = "Method"
	$g_hOut = FileOpen($g_sOUTPUT_DIR & "methods.htm", BitOR($FO_OVERWRITE, $FO_UTF8)) ;overwrite mode
	$g_sINPUT_DIR = IniRead($TXT2HTM_INI, "Input", "methods", "ERR")
	; ### Added this to determine the link to the htm file
	$g_sHtmlDir = IniRead($TXT2HTM_INI, "Output", "methods", "ERR")
	genFile()
EndIf

Exit

Func OnQuit()
	_OutputWaitClosed() ; ### When run in standalone
EndFunc   ;==>OnQuit

Func genFile()
	put('<!DOCTYPE html>')
	put('<html>')
	put('<head>')
	put('  <title>' & $g_sRefType & "s" & '</title>')
	put('  <meta charset="utf-8">')
	If $g_iReGen_AutoItX Then
		put('  <link href="../css/default.css" rel="stylesheet">')
	Else
		put('  <link href="css/default.css" rel="stylesheet">')
	EndIf
	put('</head>')
	put('<body>')
	put('<h1>' & $g_sRefType & ' Reference</h1>')
	If $g_iReGen_AutoItX Then
		put('<p>Below is a complete list of the ' & StringLower($g_sRefType) & 's available in AutoItX.')
	Else
		put('<p>Below is a complete list of the ' & StringLower($g_sRefType) & 's available in AutoIt.')
	EndIf
	put('Click on a ' & StringLower($g_sRefType) & ' name for a detailed description.</p>')
	If $g_sRefTypeNotes Then
		put('<p>See <a href="' & $g_sRefTypeNotes & '.htm">' & $g_sRefType & ' Notes</a> for details on usage.')
	EndIf
	put('<p>&nbsp;</p>')
	put('')
	put('<table>')
	put('<tr>')
	put('  <th style="width:25%">' & $g_sRefType & '</th>')
	put('  <th style="width:75%">Description</th>')
	put('</tr>')

	Local $aFileList = _FileListToArray($g_sINPUT_DIR, "*.txt", $FLTA_FILES)
	If @error Then
		MsgBox($MB_SYSTEMMODAL, "Error", "Unable to locate any text files.")
		Exit
	EndIf

	Local $sFilePath, $sLine
	$g_sHtmlDir = StringReplace($g_sHtmlDir, '\', '/') ; Replace to determine the link of the htm file.

	For $i = 1 To $aFileList[0] ; Loop through the filelist.
		If StringInStr($aFileList[$i], "$tag") Then ContinueLoop
		$sFilePath = $g_sINPUT_DIR & $aFileList[$i]
		$g_hIn = FileOpen($sFilePath) ; Read mode

		; Loop thru each line in the current input file
		$sLine = FileReadLine($g_hIn)
		While Not @error
			If StringInStr($sLine, "###" & $g_sRefType & "###") Then
				put('<tr>')
				$sLine = StringStripWS(FileReadLine($g_hIn), BitOR($STR_STRIPLEADING, $STR_STRIPTRAILING))
				;Get 1st non-blank line; (assume it's the keyword/function name)
				While $sLine = ""
					$sLine = StringStripWS(FileReadLine($g_hIn), BitOR($STR_STRIPLEADING, $STR_STRIPTRAILING))
				WEnd
				$aFileList[$i] = StringReplace($aFileList[$i], ".txt", ".htm")
				; ### Change 2 line to determine the link to the htm file
				;$sFileName = StringLower($g_sRefType) & 's\' & $sFileName
				;$sFileName = @@WorkingDir & '\' & $g_sHtmlDir & $sFileName
				$aFileList[$i] = StringTrimLeft($g_sHtmlDir, StringLen($g_sOUTPUT_DIR)) & $aFileList[$i]
				put('  <td><a href="' & $aFileList[$i] & '">' & $sLine & '</a></td>')

				$sLine = FileReadLine($g_hIn)
				While Not @error
					If StringInStr($sLine, "###Description###") Then
						$sLine = StringStripWS(FileReadLine($g_hIn), BitOR($STR_STRIPLEADING, $STR_STRIPTRAILING))
						;Get 1st non-blank line; (assume it's the description)
						While $sLine = ""
							$sLine = StringStripWS(FileReadLine($g_hIn), BitOR($STR_STRIPLEADING, $STR_STRIPTRAILING))
						WEnd
						put('  <td>' & $sLine & '</td>')
					EndIf
					$sLine = FileReadLine($g_hIn)
				WEnd
				put('</tr>')

				ExitLoop
			EndIf
			$sLine = FileReadLine($g_hIn)
		WEnd

		FileClose($g_hIn)
	Next

	put('</table>')
	put('')
	put('</body>')
	put('</html>')

	FileClose($g_hOut)
	Return
EndFunc   ;==>genFile

Func GenSubRefs()
	Local $sCATALOG_TOC = $g_sINPUT_DIR & "Categories.toc"

	Local $hList = FileOpen($sCATALOG_TOC) ;readmode
	If $hList = -1 Then
		MsgBox($MB_SYSTEMMODAL, "Error", $sCATALOG_TOC & " cannot be read")
		Exit
	EndIf

	Local $sLine = "", $aField, $sRefType = "", $sSubMgt = "", $iSplit
	$g_hOut = -1
	While 1
		If $sLine = "" Then $sLine = FileReadLine($hList)
		If @error Then ExitLoop

		$aField = StringSplit($sLine, '|')
		$sLine = ""
		If $aField[0] >= 3 Then
			If $sRefType <> $aField[1] Then
				PutFooter()
				$sRefType = $aField[1]
				$g_hOut = FileOpen($g_sOUTPUT_DIR & "libfunctions\" & $sRefType & ".htm", BitOR($FO_OVERWRITE, $FO_UTF8)) ;overwrite mode
				PutHeader($sRefType)
			EndIf

			$iSplit = StringInStr($aField[1], " ")
			$aField[1] = StringLeft($aField[1], $iSplit)

			If $g_hOut <> -1 Then
				If $aField[2] <> $sSubMgt Then
					$sSubMgt = $aField[2]
					put('      <li><a href="' & $aField[1] & $sSubMgt & '.htm">' & $sSubMgt & '</a></li>')
				EndIf

				If $aField[0] >= 4 Then
					Local $hSav = $g_hOut
					$g_hOut = FileOpen($g_sHtmlDir & $aField[1] & $aField[2] & ".htm", BitOR($FO_OVERWRITE, $FO_UTF8)) ;overwrite mode
					$sLine = GenSubSubRef($hList, $aField[1], $aField[2], $aField[3])
					FileClose($g_hOut)
					$g_hOut = $hSav
				EndIf
			EndIf
		EndIf
	WEnd
	PutFooter()

	FileClose($hList)
	Return
EndFunc   ;==>GenSubRefs

Func PutHeader($sRefType)
	put('<!DOCTYPE html>')
	put('<html>')
	put('<head>')
	put('  <title>' & $sRefType & '</title>')
	put('  <meta charset="utf-8">')
	put('  <link href="../css/default.css" rel="stylesheet">')
	put('</head>')
	put('')
	put('<body>')
	put('<h1>' & $sRefType & '</h1>')
	put('<ul>')
EndFunc   ;==>PutHeader

Func PutFooter()
	If $g_hOut = -1 Then Return
	put('</ul>')
	put('')
	put('</body>')
	put('</html>')
	FileClose($g_hOut)
	$g_hOut = -1
EndFunc   ;==>PutFooter

Func GenSubSubRef($hList, $sPrefix, $sRefType, $sSubMgt)
	PutHeader($sRefType)
	put('      <li><a href="' & $sPrefix & $sSubMgt & '.htm">' & $sSubMgt & '</a></li>')

	Local $sLine, $aField
	While 1
		$sLine = FileReadLine($hList)
		If @error Then ExitLoop
		If StringInStr($sLine, $sRefType) Then
			$aField = StringSplit($sLine, '|')
			If $aField[3] <> $sSubMgt Then
				$sSubMgt = $aField[3]
				put('      <li><a href="' & $sPrefix & $sSubMgt & '.htm">' & $sSubMgt & '</a></li>')
			EndIf
		Else
			ExitLoop
		EndIf
	WEnd

	PutFooter()

	Return $sLine
EndFunc   ;==>GenSubSubRef

;------------------------------------------------------------------------------
; Write a new line to the output file
;------------------------------------------------------------------------------
Func put($sLineToWrite)
	FileWriteLine($g_hOut, $sLineToWrite)
EndFunc   ;==>put
