; ===================================================================
; Project: Help File Library
; Description: This file contains TXT doc related functions.
; Author: guinness
; ===================================================================
#include-once

#include <String.au3>

#include '..\MiscLib.au3'

Global Const $HELPFILE_GUID = '3E7D16CA-1847-11E4-B1CF-5506E3E253B6'
; I guess the map data structure would be ideal, IF the internal order doesn't change.
Global Enum $HELPFILE_BUILDDATA, _
		$HELPFILE_FILEDATA, _
		$HELPFILE_EXAMPLEPATH, _
		$HELPFILE_FILEDIR, _
		$HELPFILE_FILEPATH, _
		$HELPFILE_FILENAME, _
		$HELPFILE_CURRENTLINE, _
		$HELPFILE_FIRSTLINE, _
		$HELPFILE_LASTLINE, _
		$HELPFILE_LINECOUNT, _
		$HELPFILE_MODIFIEDDATE, _
		$HELPFILE_UBOUND, _
		$HELPFILE_ID, _
		$HELPFILE_ISTAG, _
		$HELPFILE_SECTIONS, _
		$HELPFILE_TYPE, _
		$HELPFILE_MAX
Global Const $HELPFILE_DETAILS = _StringRepeat(@CRLF, $HELPFILE_MAX)
Global Const $HELPFILE_SECONDLINE = 1
Global Enum $HELPFILE_TYPE_NONE, _
		$HELPFILE_TYPE_AUTOITX, _
		$HELPFILE_TYPE_KEYWORD, _
		$HELPFILE_TYPE_NATIVE, _
		$HELPFILE_TYPE_USERDEFINED
Global Enum $HELPFILE_ERROR_NONE, _
		$HELPFILE_ERROR_INVALIDSECTION
Global Enum $HELPFILE_SECTION_KEY, _
		$HELPFILE_SECTION_VALUE, _
		$HELPFILE_SECTION_MAX

Func _HelpFile($sFilePath, $sExamplePath)
	Local $sData = FileRead($sFilePath)
	; If @error Then Return Null ; File doesn't exist. ; Not required.

	Local Const $STR_STRINGSPLIT = BitOR($STR_ENTIRESPLIT, $STR_NOCOUNT)
	Local $aHelpFile = StringSplit($HELPFILE_DETAILS & $sData, @CRLF, $STR_STRINGSPLIT)
	$aHelpFile[$HELPFILE_UBOUND] = UBound($aHelpFile)
	__HelpFile_SetExamplePath($aHelpFile, $sExamplePath)
	$aHelpFile[$HELPFILE_FILEPATH] = $sFilePath

	$aHelpFile[$HELPFILE_FILENAME] = StringInStr($aHelpFile[$HELPFILE_FILEPATH], '\', $STR_NOCASESENSEBASIC, -1) ; Should be a different variable to store the position of \, but lets double up for speed.
	$aHelpFile[$HELPFILE_FILEDIR] = StringLeft($aHelpFile[$HELPFILE_FILEPATH], $aHelpFile[$HELPFILE_FILENAME] - 1) ; Trim the filespec.

	$aHelpFile[$HELPFILE_FILENAME] = StringTrimLeft($aHelpFile[$HELPFILE_FILEPATH], $aHelpFile[$HELPFILE_FILENAME]) ; Trim the filepath by using the position of \.
	$aHelpFile[$HELPFILE_FILENAME] = StringLeft($aHelpFile[$HELPFILE_FILENAME], StringInStr($aHelpFile[$HELPFILE_FILENAME], '.', $STR_NOCASESENSEBASIC, -1) - 1) ; Trim the extension.

	Local Const $FT_STRING = 1
	$aHelpFile[$HELPFILE_MODIFIEDDATE] = Int(FileGetTime($aHelpFile[$HELPFILE_FILEPATH], $FT_MODIFIED, $FT_STRING))
	; If @error THen $aHelpFile[$HELPFILE_MODIFIEDDATE] = Null ; Not required.

	$aHelpFile[$HELPFILE_FIRSTLINE] = $HELPFILE_MAX
	$aHelpFile[$HELPFILE_LASTLINE] = $aHelpFile[$HELPFILE_UBOUND] - 1
	$aHelpFile[$HELPFILE_LINECOUNT] = $aHelpFile[$HELPFILE_UBOUND] - $HELPFILE_MAX

	; This can be changed only! It's the current line in the internal object.
	$aHelpFile[$HELPFILE_CURRENTLINE] = $aHelpFile[$HELPFILE_FIRSTLINE] - 1 ; Minus 1, so when the likes of _HelpFile_GetNextLine() are first called it retrieves the first line.

	$aHelpFile[$HELPFILE_FILEDATA] = $sData ; Only for comparison during the re-building stage.
	$aHelpFile[$HELPFILE_BUILDDATA] = $aHelpFile[$HELPFILE_FILEDATA] ; Correctly formatted version of the text file.

	_ConvertEOLToCRLF($aHelpFile[$HELPFILE_BUILDDATA])
	_StripTrailingWhitespace($aHelpFile[$HELPFILE_BUILDDATA])
	; _StripEmptyLinesDouble($aHelpFile[$HELPFILE_BUILDDATA])

	; $aHelpFile[$HELPFILE_ISTAG] = StringRegExp($aHelpFile[$HELPFILE_FILENAME], '^\$tag') > 0
	$aHelpFile[$HELPFILE_ISTAG] = StringRegExp($aHelpFile[$HELPFILE_BUILDDATA], '^\h*###Structure Name###\R') > 0
	$aHelpFile[$HELPFILE_TYPE] = $HELPFILE_TYPE_NONE

	$aHelpFile[$HELPFILE_SECTIONS] = 0
	If __HelpFile_GenerateSections($aHelpFile) Then
		$aHelpFile[$HELPFILE_ID] = $HELPFILE_GUID ; Set the GUID so as to check later on that in fact it is a valid help file handle.
		_HelpFile_BuildSectionData($aHelpFile)
	EndIf

	Return $aHelpFile
EndFunc   ;==>_HelpFile

Func _HelpFile_BuildSave(ByRef $aHelpFile)
	_HelpFile_BuildSectionData($aHelpFile)
	Local $sData = _HelpFile_GetBuildData($aHelpFile), $sDataOriginal = _HelpFile_GetData($aHelpFile), $sFilePath = _HelpFile_GetFilePath($aHelpFile)
	Return _FileWrite_IfDiff($sFilePath, $sFilePath, $sDataOriginal, $sData)
EndFunc   ;==>_HelpFile_BuildSave

Func _HelpFile_BuildSections(ByRef $aHelpFile)
	Local $bReturn = False
	If __HelpFile_IsAPI($aHelpFile) Then
		$bReturn = __HelpFile_GenerateSections($aHelpFile)
		If $bReturn Then
			_HelpFile_BuildSectionData($aHelpFile)
		EndIf
	EndIf
	Return $bReturn
EndFunc   ;==>_HelpFile_BuildSections

Func _HelpFile_BuildSectionData(ByRef $aHelpFile)
	Local $bReturn = False
	If __HelpFile_IsAPI($aHelpFile) Then
		Local $sData = ''
		Local $aKeys = $aHelpFile[$HELPFILE_SECTIONS].Keys()
		For $sKey In $aKeys
			$sData &= $sKey & @CRLF & $aHelpFile[$HELPFILE_SECTIONS].Item($sKey) & @CRLF & @CRLF ; & @CRLF
		Next

		If Not (StringStripWS($sData, $STR_STRIPALL) == '') Then
			$bReturn = True
			; Removed as it's no longer need where there are two EOLs for these sections only.
			; $sData = StringRegExpReplace($sData, '(?is:(###(?:Description|Fields|Function|Keyword|Macro|Operator|Parameters|Structure Name|User Defined Function)###\R.+?)\R{3}(?=###[^#]+?###\R))', '\1' & @CRLF & @CRLF)

			; Remove empty lines at the end of a file.
			_StripEmptyLinesEndOfFile($sData)

			; Append @CRLF.
			$sData &= @CRLF

			$bReturn = _HelpFile_SetBuildData($aHelpFile, $sData)
			If $bReturn Then
				__HelpFile_GenerateSections($aHelpFile) ; Generate sections.
			EndIf
		EndIf
	EndIf
	Return $bReturn
EndFunc   ;==>_HelpFile_BuildSectionData

Func _HelpFile_GetBuildData(ByRef $aHelpFile)
	Return (__HelpFile_IsAPI($aHelpFile) ? $aHelpFile[$HELPFILE_BUILDDATA] : Null)
EndFunc   ;==>_HelpFile_GetBuildData

Func _HelpFile_GetCurrentLine(ByRef $aHelpFile)
	Return (__HelpFile_IsAPI($aHelpFile) And __HelpFile_IsInBounds($aHelpFile, $aHelpFile[$HELPFILE_CURRENTLINE]) ? $aHelpFile[$aHelpFile[$HELPFILE_CURRENTLINE]] : Null)
EndFunc   ;==>_HelpFile_GetCurrentLine

Func _HelpFile_GetCurrentLineValue(ByRef $aHelpFile)
	Return (__HelpFile_IsAPI($aHelpFile) And __HelpFile_IsInBounds($aHelpFile, $aHelpFile[$HELPFILE_CURRENTLINE]) ? ($aHelpFile[$HELPFILE_CURRENTLINE] - $HELPFILE_MAX) + 1 : 0) ; The current line is calculated by the current line minus the section reserved for info storage. The +1 is because it returns 0 and visually we're used to 1, 2, 3...
EndFunc   ;==>_HelpFile_GetCurrentLineValue

Func _HelpFile_GetData(ByRef $aHelpFile)
	Return (__HelpFile_IsAPI($aHelpFile) ? $aHelpFile[$HELPFILE_FILEDATA] : Null)
EndFunc   ;==>_HelpFile_GetData

Func _HelpFile_GetExamplePath(ByRef $aHelpFile)
	Return (__HelpFile_IsAPI($aHelpFile) ? $aHelpFile[$HELPFILE_EXAMPLEPATH] : Null)
EndFunc   ;==>_HelpFile_GetExamplePath

Func _HelpFile_GetFileDir(ByRef $aHelpFile)
	Return (__HelpFile_IsAPI($aHelpFile) ? $aHelpFile[$HELPFILE_FILEDIR] : Null)
EndFunc   ;==>_HelpFile_GetFileDir

Func _HelpFile_GetFileName(ByRef $aHelpFile)
	Return (__HelpFile_IsAPI($aHelpFile) ? $aHelpFile[$HELPFILE_FILENAME] : Null)
EndFunc   ;==>_HelpFile_GetFileName

Func _HelpFile_GetFilePath(ByRef $aHelpFile)
	Return (__HelpFile_IsAPI($aHelpFile) ? $aHelpFile[$HELPFILE_FILEPATH] : Null)
EndFunc   ;==>_HelpFile_GetFilePath

Func _HelpFile_GetIndexToSection(ByRef $aHelpFile, $iIndex)
	Local $sSection = ''
	If __HelpFile_IsAPI($aHelpFile) Then
		$iIndex = Int($iIndex)
		Local $aKeys = $aHelpFile[$HELPFILE_SECTIONS].Keys()
		If $iIndex >= 0 And $iIndex < UBound($aKeys) Then $sSection = $aKeys[$iIndex]
	EndIf
	Return $sSection
EndFunc   ;==>_HelpFile_GetIndexToSection

Func _HelpFile_GetLineCount(ByRef $aHelpFile)
	Return (__HelpFile_IsAPI($aHelpFile) ? $aHelpFile[$HELPFILE_LINECOUNT] : 0)
EndFunc   ;==>_HelpFile_GetLineCount

Func _HelpFile_GetModifiedDate(ByRef $aHelpFile)
	Return (__HelpFile_IsAPI($aHelpFile) ? $aHelpFile[$HELPFILE_MODIFIEDDATE] : 0)
EndFunc   ;==>_HelpFile_GetModifiedDate

Func _HelpFile_GetNextLine(ByRef $aHelpFile)
	Local $sReturn = Null
	If __HelpFile_IsAPI($aHelpFile) And __HelpFile_IsInBounds($aHelpFile, $aHelpFile[$HELPFILE_CURRENTLINE] + 1) Then
		$aHelpFile[$HELPFILE_CURRENTLINE] += 1
		$sReturn = _HelpFile_GetCurrentLine($aHelpFile)
	EndIf
	Return $sReturn
EndFunc   ;==>_HelpFile_GetNextLine

Func _HelpFile_GetPreviousLine(ByRef $aHelpFile)
	Local $sReturn = Null
	If __HelpFile_IsAPI($aHelpFile) And __HelpFile_IsInBounds($aHelpFile, $aHelpFile[$HELPFILE_CURRENTLINE] - 1) Then
		$aHelpFile[$HELPFILE_CURRENTLINE] -= 1
		$sReturn = _HelpFile_GetCurrentLine($aHelpFile)
	EndIf
	Return $sReturn
EndFunc   ;==>_HelpFile_GetPreviousLine

Func _HelpFile_GetSection(ByRef $aHelpFile, $sSection)
	Local $iError = $HELPFILE_ERROR_NONE, _
			$sData = Null
	If __HelpFile_IsAPI($aHelpFile) Then
		__HelpFile_FormatSection($sSection)
		If $aHelpFile[$HELPFILE_SECTIONS].Exists($sSection) Then
			$sData = $aHelpFile[$HELPFILE_SECTIONS].Item($sSection)
		Else
			$iError = $HELPFILE_ERROR_INVALIDSECTION
		EndIf
	EndIf
	Return SetError($iError, 0, $sData)
EndFunc   ;==>_HelpFile_GetSection

Func _HelpFile_GetSectionArray(ByRef $aHelpFile)
	Local $aReturn[0][0] ; Empty array.
	If __HelpFile_IsAPI($aHelpFile) Then
		Local $aKeys = $aHelpFile[$HELPFILE_SECTIONS].Keys()
		Local $aArray[UBound($aKeys)][$HELPFILE_SECTION_MAX], _
				$iIndex = 0
		For $sKey In $aKeys
			$aArray[$iIndex][$HELPFILE_SECTION_KEY] = $sKey
			$aArray[$iIndex][$HELPFILE_SECTION_VALUE] = $aHelpFile[$HELPFILE_SECTIONS].Item($sKey)
			$iIndex += 1
		Next
		$aReturn = $aArray
		$aArray = 0
	EndIf
	Return $aReturn
EndFunc   ;==>_HelpFile_GetSectionArray

Func _HelpFile_GetSectionToIndex(ByRef $aHelpFile, $sSection)
	Local $iIndex = -1
	If __HelpFile_IsAPI($aHelpFile) Then
		__HelpFile_FormatSection($sSection)
		Local $aKeys = $aHelpFile[$HELPFILE_SECTIONS].Keys()
		If UBound($aKeys) Then
			$iIndex = 0
			For $sKey In $aKeys
				If $sSection = $sKey Then
					ExitLoop
				EndIf
				$iIndex += 1
			Next
		EndIf
	EndIf
	Return $iIndex
EndFunc   ;==>_HelpFile_GetSectionToIndex

Func _HelpFile_IsAutoItX(ByRef $aHelpFile)
	Return (__HelpFile_IsAPI($aHelpFile) ? ($aHelpFile[$HELPFILE_TYPE] = $HELPFILE_TYPE_AUTOITX) : False)
EndFunc   ;==>_HelpFile_IsAutoItX

Func _HelpFile_IsFileEnd(ByRef $aHelpFile)
	Return __HelpFile_IsAPI($aHelpFile) And $aHelpFile[$HELPFILE_CURRENTLINE] = $aHelpFile[$HELPFILE_LASTLINE]
EndFunc   ;==>_HelpFile_IsFileEnd

Func _HelpFile_IsHelpFile(ByRef $aHelpFile)
	Return __HelpFile_IsAPI($aHelpFile) And __HelpFile_IsValidType($aHelpFile)
EndFunc   ;==>_HelpFile_IsHelpFile

Func _HelpFile_IsKeyword(ByRef $aHelpFile)
	Return (__HelpFile_IsAPI($aHelpFile) ? ($aHelpFile[$HELPFILE_TYPE] = $HELPFILE_TYPE_KEYWORD) : False)
EndFunc   ;==>_HelpFile_IsKeyword

Func _HelpFile_IsNative(ByRef $aHelpFile)
	Return (__HelpFile_IsAPI($aHelpFile) ? ($aHelpFile[$HELPFILE_TYPE] = $HELPFILE_TYPE_NATIVE) : False)
EndFunc   ;==>_HelpFile_IsNative

Func _HelpFile_IsSectionExists(ByRef $aHelpFile, $sSection)
	_HelpFile_GetSection($aHelpFile, $sSection)
	Return @error = $HELPFILE_ERROR_NONE
EndFunc   ;==>_HelpFile_IsSectionExists

Func _HelpFile_IsTag(ByRef $aHelpFile)
	Return (__HelpFile_IsAPI($aHelpFile) ? $aHelpFile[$HELPFILE_ISTAG] : Null)
EndFunc   ;==>_HelpFile_IsTag

Func _HelpFile_IsUserDefined(ByRef $aHelpFile)
	Return (__HelpFile_IsAPI($aHelpFile) ? ($aHelpFile[$HELPFILE_TYPE] = $HELPFILE_TYPE_USERDEFINED) : False)
EndFunc   ;==>_HelpFile_IsUserDefined

Func _HelpFile_RemoveSection(ByRef $aHelpFile, $sSection)
	Local $bReturn = False
	If __HelpFile_IsAPI($aHelpFile) Then
		__HelpFile_FormatSection($sSection)
		If $aHelpFile[$HELPFILE_SECTIONS].Exists($sSection) Then
			$bReturn = True
			$aHelpFile[$HELPFILE_SECTIONS].Remove($sSection)
		EndIf
	EndIf
	Return $bReturn
EndFunc   ;==>_HelpFile_RemoveSection

Func _HelpFile_SetAsAutoItX(ByRef $aHelpFile)
	Return __HelpFile_SetAsType($aHelpFile, $HELPFILE_TYPE_AUTOITX)
EndFunc   ;==>_HelpFile_SetAsAutoItX

Func _HelpFile_SetAsKeyword(ByRef $aHelpFile)
	Return __HelpFile_SetAsType($aHelpFile, $HELPFILE_TYPE_KEYWORD)
EndFunc   ;==>_HelpFile_SetAsKeyword

Func _HelpFile_SetAsNative(ByRef $aHelpFile)
	Return __HelpFile_SetAsType($aHelpFile, $HELPFILE_TYPE_NATIVE)
EndFunc   ;==>_HelpFile_SetAsNative

Func _HelpFile_SetAsUserDefined(ByRef $aHelpFile)
	Return __HelpFile_SetAsType($aHelpFile, $HELPFILE_TYPE_USERDEFINED)
EndFunc   ;==>_HelpFile_SetAsUserDefined

Func _HelpFile_SetBuildData(ByRef $aHelpFile, $sData)
	Local $bReturn = False
	If __HelpFile_IsAPI($aHelpFile) Then
		If Not ($aHelpFile[$HELPFILE_BUILDDATA] == $sData) Then
			$bReturn = True
			$aHelpFile[$HELPFILE_BUILDDATA] = $sData
		EndIf
	EndIf
	Return $bReturn
EndFunc   ;==>_HelpFile_SetBuildData

Func _HelpFile_SetData(ByRef $aHelpFile, $sData)
	Local $bReturn = False
	If __HelpFile_IsAPI($aHelpFile) Then
		If Not ($aHelpFile[$HELPFILE_FILEDATA] == $sData) Then
			$bReturn = True
			$aHelpFile[$HELPFILE_FILEDATA] = $sData
		EndIf
	EndIf
	Return $bReturn
EndFunc   ;==>_HelpFile_SetData

Func _HelpFile_SetExamplePath(ByRef $aHelpFile, $sFilePath)
	Return __HelpFile_IsAPI($aHelpFile) And __HelpFile_SetExamplePath($aHelpFile, $sFilePath)
EndFunc   ;==>_HelpFile_SetExamplePath

Func _HelpFile_SetSection(ByRef $aHelpFile, $sSection, $sData)
	Local $bReturn = False
	If __HelpFile_IsAPI($aHelpFile) Then
		__HelpFile_FormatSection($sSection)
		If $aHelpFile[$HELPFILE_SECTIONS].Exists($sSection) Then
			$bReturn = True
			$aHelpFile[$HELPFILE_SECTIONS].Item($sSection) = $sData
		EndIf
	EndIf
	Return $bReturn
EndFunc   ;==>_HelpFile_SetSection

Func __HelpFile_FormatSection(ByRef $sSection)
	If Not __HelpFile_IsSection($sSection) Then
		$sSection = '###' & $sSection & '###' ; Format to the correct section syntax.
	EndIf
EndFunc   ;==>__HelpFile_FormatSection

Func __HelpFile_IsAPI(ByRef $aHelpFile)
	Return UBound($aHelpFile) >= $HELPFILE_MAX And $aHelpFile[$HELPFILE_ID] = $HELPFILE_GUID
EndFunc   ;==>__HelpFile_IsAPI

Func __HelpFile_IsInBounds(ByRef $aHelpFile, $iIndex)
	Return $iIndex >= $aHelpFile[$HELPFILE_FIRSTLINE] And $iIndex <= $aHelpFile[$HELPFILE_LASTLINE]
EndFunc   ;==>__HelpFile_IsInBounds

Func __HelpFile_IsSection($sSection)
	Return StringRegExp($sSection, '^###[^#]+?###$') > 0
EndFunc   ;==>__HelpFile_IsSection

Func __HelpFile_IsValidType(ByRef $aHelpFile)
	Return $aHelpFile[$HELPFILE_TYPE] = $HELPFILE_TYPE_AUTOITX Or $aHelpFile[$HELPFILE_TYPE] = $HELPFILE_TYPE_KEYWORD Or $aHelpFile[$HELPFILE_TYPE] = $HELPFILE_TYPE_NATIVE Or $aHelpFile[$HELPFILE_TYPE] = $HELPFILE_TYPE_USERDEFINED
EndFunc   ;==>__HelpFile_IsValidType

Func __HelpFile_GenerateSections(ByRef $aHelpFile)
	Local $bReturn = False
	$aHelpFile[$HELPFILE_SECTIONS] = 0 ; Clear the previous sections.
	If _AssociativeArray_Startup($aHelpFile[$HELPFILE_SECTIONS], False) Then
		$bReturn = True
		Local $aSections = StringRegExp(@CRLF & $aHelpFile[$HELPFILE_BUILDDATA] & @CRLF, '(?s:\h*\R(###[^#]+?###)\h*\R(.*?(?=(?:\R\h*)+(?:###[^#]+?###|$))))', $STR_REGEXPARRAYGLOBALMATCH)
		If Not @error Then
			Local Enum $SECTION_NAME, $SECTION_CONTENT
			Local $sSection = ''
			For $i = 0 To UBound($aSections) - 1 Step 2
				$sSection = $aSections[$i + $SECTION_CONTENT]
				; Strip empty lines at the start of a section.
				_StripEmptyLinesStartOfFile($sSection)
				; Strip empty lines at the end of a section.
				; _StripEmptyLinesEndOfFile($sSection)
				$aHelpFile[$HELPFILE_SECTIONS].Item($aSections[$i + $SECTION_NAME]) = $sSection
				; ConsoleWrite($aSections[$i + $SECTION_NAME] & @CRLF & $aSections[$i + $SECTION_CONTENT] & @CRLF & @CRLF)
			Next
		EndIf
	EndIf
	Return $bReturn
EndFunc   ;==>__HelpFile_GenerateSections

Func __HelpFile_SetAsType(ByRef $aHelpFile, $iType)
	Local $bReturn = False
	If __HelpFile_IsAPI($aHelpFile) Then
		$bReturn = True
		$aHelpFile[$HELPFILE_TYPE] = $iType
	EndIf
	Return $bReturn
EndFunc   ;==>__HelpFile_SetAsType

Func __HelpFile_SetExamplePath(ByRef $aHelpFile, $sFilePath)
	Local $bReturn = False
	If FileExists($sFilePath) Then
		$bReturn = True
		$aHelpFile[$HELPFILE_EXAMPLEPATH] = $sFilePath
	EndIf
	Return $bReturn
EndFunc   ;==>__HelpFile_SetExamplePath
