; ===================================================================
; Project: Misc Library
; Description: This file contains shared misc functions.
; Author: guinness
; ===================================================================
#include-once
#include <Array.au3>
#include <StringConstants.au3>
#include <WinAPIFiles.au3>

Global Const $ASCII_ACK = Chr(6)
#cs
	##None###
	###Method###
	###Function###
	###Keyword###
	###Macro###
	###Operator###
	###Structure Name###
	###User Defined Function###
#ce
Global Enum $HELP_TYPE_NONE, _
		$HELP_TYPE_AUTOITX, _
		$HELP_TYPE_KEYWORD, _
		$HELP_TYPE_MACRO, _
		$HELP_TYPE_NATIVE, _
		$HELP_TYPE_OPERATOR, _
		$HELP_TYPE_STRUCTURE, _
		$HELP_TYPE_USERDEFINED

Global Const $HELP_SECTION_NONE = '###SectionNone###', _
		$HELP_SECTION_DESCRIPTION = '###Description###', _
		$HELP_SECTION_EXAMPLE = '###Example###', _
		$HELP_SECTION_EXPERIMENTAL = '###Experimental###', _
		$HELP_SECTION_FIELDS = '###Fields###', _
		$HELP_SECTION_FUNCTION = '###Function###', _
		$HELP_SECTION_KEYWORD = '###Keyword###', _
		$HELP_SECTION_MACRO = '###Macro###', _
		$HELP_SECTION_METHOD = '###Method###', _
		$HELP_SECTION_OPERATOR = '###Operator###', _
		$HELP_SECTION_PARAMETERS = '###Parameters###', _
		$HELP_SECTION_RELATED = '###Related###', _
		$HELP_SECTION_REMARKS = '###Remarks###', _
		$HELP_SECTION_RETURNVALUE = '###ReturnValue###', _
		$HELP_SECTION_SEEALSO = '###See Also###', _
		$HELP_SECTION_STRUCTURENAME = '###Structure Name###', _
		$HELP_SECTION_SYNTAX = '###Syntax###', _
		$HELP_SECTION_USERDEFINED = '###User Defined Function###'

Global Const $FC_OVERWRITE_CREATE = BitOR($FC_CREATEPATH, $FC_OVERWRITE)
Global Const $FO_OVERWRITE_CREATE = BitOR($FO_CREATEPATH, $FO_OVERWRITE)
Global Enum $MACROLISTTOARRAY_NAME, _
		$MACROLISTTOARRAY_DESCRIPTION, _
		$MACROLISTTOARRAY_MAX
Global Const $STR_REPLACE_ONCE = 1

; ------------------------------------------------------------------------------
; Associative array.
; ------------------------------------------------------------------------------
Func _AssociativeArray_Startup(ByRef $aArray, $bIsCaseSensitive = False) ; Idea from MilesAhead.
	Local $bReturn = False
	$aArray = ObjCreate('Scripting.Dictionary')
	If IsObj($aArray) Then
		$aArray.CompareMode = ($bIsCaseSensitive ? 0 : 1)
		$bReturn = True
	EndIf
	Return $bReturn
EndFunc   ;==>_AssociativeArray_Startup

; ------------------------------------------------------------------------------
; Convert & to &amp;.
; ------------------------------------------------------------------------------
Func _ConvertAMPSymbol(ByRef $sData)
	$sData = StringRegExpReplace($sData, '&(?!(?:amp|apos|gt|lt|nbsp|quot);)', '&amp;')
EndFunc   ;==>_ConvertAMPSymbol

; ------------------------------------------------------------------------------
; Escape symbols to HTML entities.
; ------------------------------------------------------------------------------
Func _ConvertSymbolsToHTMLEntity(ByRef $sData)
	_ConvertAMPSymbol($sData)
	$sData = StringReplace($sData, "'", '&apos;')
	$sData = StringReplace($sData, '>', '&gt;')
	$sData = StringReplace($sData, '<', '&lt;')
	$sData = StringReplace($sData, '"', '&quot;')
EndFunc   ;==>_ConvertSymbolsToHTMLEntity

; ------------------------------------------------------------------------------
; Escape HTML entities to symbols.
; ------------------------------------------------------------------------------
Func _ConvertHTMLEntityToSymbols(ByRef $sData)
	$sData = StringReplace($sData, '&amp;', '&')
	$sData = StringReplace($sData, '&apos;', "'")
	$sData = StringReplace($sData, '&gt;', '>')
	$sData = StringReplace($sData, '&lt;', '<')
	$sData = StringReplace($sData, '&quot;', '"')
EndFunc   ;==>_ConvertHTMLEntityToSymbols

; ------------------------------------------------------------------------------
; Convert @LF or @CR to @CRLF.
; ------------------------------------------------------------------------------
Func _ConvertEOLToCRLF(ByRef $sData)
	$sData = StringRegExpReplace($sData, '\R', @CRLF) ; By Ascend4nt.
EndFunc   ;==>_ConvertEOLToCRLF

; ------------------------------------------------------------------------------
; Overwrite a file without the need to delete it first.
; ------------------------------------------------------------------------------
Func _FileWrite($sFilePath, $sData, $iFileEncoding = 0)
	Local $bReturn = False, _
			$hFileOpen = FileOpen($sFilePath, BitOR($FO_OVERWRITE_CREATE, $iFileEncoding))
	If $hFileOpen > -1 Then
		$bReturn = FileWrite($hFileOpen, $sData) > 0
		FileClose($hFileOpen)
	EndIf
	Return $bReturn
EndFunc   ;==>_FileWrite

; ------------------------------------------------------------------------------
; Overwrite a file if the data has changed. (Case-sensitive.)
; ------------------------------------------------------------------------------
Func _FileWrite_IfDiff($sFilePathOld, $sFilePathNew, ByRef $sDataOriginal, ByRef $sDataNew)
	Local $bReturn = False
	If Not ($sDataOriginal == $sDataNew) Then
		$bReturn = _FileWrite($sFilePathNew, $sDataNew, FileGetEncoding($sFilePathOld))
	EndIf
	Return $bReturn
EndFunc   ;==>_FileWrite_IfDiff

; ------------------------------------------------------------------------------
; Retrieve the full path of a relative path.
; ------------------------------------------------------------------------------
Func _GetFullPath($sRelativePath, $sBasePath = @WorkingDir)
	Local Const $bSetWorkingDir = Not ($sBasePath = @WorkingDir), $sWorkingDir = @WorkingDir
	If $bSetWorkingDir Then FileChangeDir($sBasePath) ; Change the working directory of the current process to the base path as _WinAPI_GetFullPathName() merges the name of the current drive and directory with the specified file name.
	$sRelativePath = _WinAPI_GetFullPathName($sRelativePath)
	If $bSetWorkingDir Then FileChangeDir($sWorkingDir) ; Reset the working directory to the previous path.
	Return $sRelativePath
EndFunc   ;==>_GetFullPath

; ------------------------------------------------------------------------------
; Get functions in a script file.
; ------------------------------------------------------------------------------
Func _GetFunctionInfo(ByRef $sData, ByRef $aArray)
	$aArray = StringRegExp(@CRLF & $sData, '(?im:^Func\h+(\w+)\h*\()', $STR_REGEXPARRAYGLOBALMATCH)
EndFunc   ;==>_GetFunctionInfo

; ------------------------------------------------------------------------------
; Macro HTML data to a 2d array.
; ------------------------------------------------------------------------------
Func _MacroListToArray(ByRef $sData, ByRef $aArray, ByRef $hAssocArray)
	_ConvertAMPSymbol($sData)

	Local $aSRE = StringRegExp($sData, '(?is:</a><strong>(\@\w+)</strong></td>\R\s*<td>(.*?)</td>)', $STR_REGEXPARRAYGLOBALMATCH)
	If Not @error Then
		Local $iUBound = UBound($aSRE)
		_AssociativeArray_Startup($hAssocArray, True)
		Local $aMerge[Ceiling($iUBound / $MACROLISTTOARRAY_MAX)][$MACROLISTTOARRAY_MAX], _ ; Returns an array that starts at 0.
				$iIndex = 0
		For $i = 0 To $iUBound - 1 Step $MACROLISTTOARRAY_MAX
			$aMerge[$iIndex][$MACROLISTTOARRAY_NAME] = $aSRE[$i + $MACROLISTTOARRAY_NAME] ; Macro
			$aMerge[$iIndex][$MACROLISTTOARRAY_DESCRIPTION] = $aSRE[$i + $MACROLISTTOARRAY_DESCRIPTION] ; Description
			$hAssocArray.Item($aMerge[$iIndex][$MACROLISTTOARRAY_NAME]) = $aMerge[$iIndex][$MACROLISTTOARRAY_DESCRIPTION]
			$iIndex += 1
		Next
		; Sort the macro array in descending order.
		_ArraySort($aMerge)
		$aArray = $aMerge
	EndIf
	Return True
EndFunc   ;==>_MacroListToArray

; ------------------------------------------------------------------------------
; Sort include lines.
; ------------------------------------------------------------------------------
Func _SortIncludes(ByRef $sData)
	If Not StringInStr($sData, '#include') Then Return False

	Local $aSRE = StringRegExp($sData, '(?im)^(#include\h["''<].*?["''>])\R', $STR_REGEXPARRAYGLOBALMATCH)
	If Not @error Then
		For $i = 0 To UBound($aSRE) - 1
			$sData = StringReplace($sData, $aSRE[$i], $ASCII_ACK, $STR_REPLACE_ONCE, $STR_CASESENSE)
		Next
		_ArraySort($aSRE)
		For $i = 0 To UBound($aSRE) - 1
			$sData = StringReplace($sData, $ASCII_ACK, $aSRE[$i], $STR_REPLACE_ONCE, $STR_CASESENSE)
		Next
	EndIf
	Return True
EndFunc   ;==>_SortIncludes

; ------------------------------------------------------------------------------
; Strip the ACK character.
; ------------------------------------------------------------------------------
Func _StripACK(ByRef $sData)
	$sData = StringReplace($sData, $ASCII_ACK, '')
EndFunc   ;==>_StripACK

; ------------------------------------------------------------------------------
; Strip comment lines.
; ------------------------------------------------------------------------------
Func _StripCommentLines(ByRef $sData, $sReplace = Default, $bCommentBlocks = True)
	If $sReplace = Default Then
		$sReplace = ''
	EndIf
	_ConvertEOLToCRLF($sData)
	$sData = StringTrimLeft(StringRegExpReplace($sData, '\R[^;"''\r\n]*(?:[^;"''\r\n]|''[^''\r\n]*''|"[^"\r\n]*")*\K;[^\r\n]*', ''), StringLen(@LF)) ; By DXRW4E. http://www.autoitscript.com/forum/topic/157255-regular-expression-challenge-for-stripping-single-comments/
	If $bCommentBlocks Then
		$sData = StringRegExpReplace($sData, '(?ims:^\h*#(?:cs|comments-start)(?!\w).+?#(?:ce|comments-end)[^\r\n]*\R)', $sReplace) ; Strip comment blocks to avoid unintended matches. By Robjong
	EndIf
	$sData = StringRegExpReplace($sData, '(?im:^#(?:(?:end)?region)(?!\w)[^\r\n]+\R)', $sReplace) ; By DXRW4E. Fixed by guinness.
	$sData = StringRegExpReplace($sData, '(?im:^#(?!autoit|force(def|ref)|ignorefunc|include(-once)?|' & _
			'noautoit|notrayicon|onautoitstartregister|au3stripper|obfuscator|pragma|requireadmin|tidy)[^\r\n]*\R)', $sReplace) ; Strip user custom comments. By guinness.
EndFunc   ;==>_StripCommentLines

; ------------------------------------------------------------------------------
; Strip empty lines.
; ------------------------------------------------------------------------------
Func _StripEmptyLines(ByRef $sData)
	$sData = StringRegExpReplace($sData, '(?m:^\h*\R)', '') ; Empty lines. By guinness.
EndFunc   ;==>_StripEmptyLines

; ------------------------------------------------------------------------------
; Remove blanks lines at the start of a file.
; ------------------------------------------------------------------------------
Func _StripEmptyLinesDouble(ByRef $sData)
	$sData = StringRegExpReplace($sData, '(?m:^\h*\R){2,}', @CRLF) ; Strip double empty lines. By guinness.
EndFunc   ;==>_StripEmptyLinesDouble

; ------------------------------------------------------------------------------
; Remove blanks lines at the end of the file. Option to append e.g. CRLF.
; ------------------------------------------------------------------------------
Func _StripEmptyLinesEndOfFile(ByRef $sData, $sAppend = '')
	$sData = StringRegExpReplace($sData, '\v+$', '') & $sAppend
EndFunc   ;==>_StripEmptyLinesEndOfFile

; ------------------------------------------------------------------------------
; Remove blanks lines at the end of the file and the ACK character. Option to append e.g. CRLF.
; ------------------------------------------------------------------------------
Func _StripEmptyLinesEndOfFileACK(ByRef $sData, $sAppend = '')
	$sData = StringRegExpReplace($sData, '\x{6}\v*$', '') & $sAppend
EndFunc   ;==>_StripEmptyLinesEndOfFileACK

; ------------------------------------------------------------------------------
; Remove blanks lines at the start of a file. Option to append e.g. CRLF.
; ------------------------------------------------------------------------------
Func _StripEmptyLinesStartOfFile(ByRef $sData, $sAppend = '')
	$sData = StringRegExpReplace($sData, '^\v+', '') & $sAppend
EndFunc   ;==>_StripEmptyLinesStartOfFile

; ------------------------------------------------------------------------------
; Merge continuation lines i.e. $sVar, _ CRLF $iNum = ""
; ------------------------------------------------------------------------------
Func _StripMerge(ByRef $sData)
	$sData = StringRegExpReplace($sData, '(?:_\h*\R\h*)', '') ; Merge continuation lines that use _. By guinness.
	Return @extended > 0
EndFunc   ;==>_StripMerge

; ------------------------------------------------------------------------------
; Strip leading/trailing whitespace.
; ------------------------------------------------------------------------------
Func _StripWhitespace(ByRef $sData)
	_StripTrailingWhitespace($sData)
	_StripLeadingWhitespace($sData)
EndFunc   ;==>_StripWhitespace

; ------------------------------------------------------------------------------
; Strip leading whitespace.
; ------------------------------------------------------------------------------
Func _StripLeadingWhitespace(ByRef $sData)
	$sData = StringRegExpReplace($sData, '\R\h+', @CRLF) ; By DXRW4E.
EndFunc   ;==>_StripLeadingWhitespace

; ------------------------------------------------------------------------------
; Strip trailing whitespace.
; ------------------------------------------------------------------------------
Func _StripTrailingWhitespace(ByRef $sData)
	$sData = StringRegExpReplace($sData, '\h+(?=\R)', '') ; By DXRW4E.
EndFunc   ;==>_StripTrailingWhitespace
