; ==============================================================================
; Generate the "OSLangCodes.htm" pages
; from the "OSLang.csv"

; Output directories are read from 'txt2htm.ini'
;
; the @OSLang.htm in Keywords dir is use to append the example
;
; 15 Nov 2012 - jpm - jpm@autoitscript.com
; ==============================================================================

#include <Array.au3>
#include <Constants.au3>

#include "..\..\_build\include\MiscLib.au3"
#include "..\..\_build\include\OutputLib.au3"

Opt("TrayIconDebug", 1)

OnAutoItExitRegister("OnQuit") ; ### when run in standalone
_OutputWindowCreate() ; ### when run in standalone

FileChangeDir(@ScriptDir)

Global $g_sAPPENDIX_OUT_DIR = IniRead("txt2htm.ini", "Output", "Appendix", "ERR")

If $g_sAPPENDIX_OUT_DIR = "ERR" Then
	MsgBox($MB_SYSTEMMODAL, "Error", "Could not read txt2htm.ini")
	Exit
EndIf

Global $g_hOut = FileOpen($g_sAPPENDIX_OUT_DIR & "OSLangCodes.htm", BitOR($FO_OVERWRITE, $FO_UTF8))
_OutputProgressWrite("OSLangCodes.htm creation...")

Local $sLangListFile = "OSLangCodes.csv"
Local $hList = FileOpen($sLangListFile)

Global $g_iLocale, $g_iLanguage, $g_iCountry, $g_sLanguage

Local $asCol = StringSplit(FileReadLine($hList), ";")
Global $g_sMSDNpage = $asCol[1]
Global $g_sMSDNhyperlink = $asCol[2]

Local $sLine = FileReadLine($hList)
While StringLeft($sLine, 1) = ";"
	$sLine = FileReadLine($hList)
WEnd

GetHeader($sLine)

Global $g_aLocale[500][3], $g_iNbLocale = 0, $g_iPrevLocale = 0
While 1
	$sLine = FileReadLine($hList)
	If @error = -1 Then ExitLoop
	GetLocale($sLine)
WEnd

; add missing Chinese entries
$g_aLocale[$g_iNbLocale][0] = "0x0404"
$g_aLocale[$g_iNbLocale][1] = GetLocaleName($g_aLocale[$g_iNbLocale][0])
$g_aLocale[$g_iNbLocale][2] = GetLocaleInfo($g_aLocale[$g_iNbLocale][0])
$g_iNbLocale += 1
$g_aLocale[$g_iNbLocale][0] = "0x0804"
$g_aLocale[$g_iNbLocale][1] = GetLocaleName($g_aLocale[$g_iNbLocale][0])
$g_aLocale[$g_iNbLocale][2] = GetLocaleInfo($g_aLocale[$g_iNbLocale][0])
$g_iNbLocale += 1

; add missing Scottish Gaelic entry
$g_aLocale[$g_iNbLocale][0] = "0x0491"
$g_aLocale[$g_iNbLocale][1] = GetLocaleName($g_aLocale[$g_iNbLocale][0])
If Not @error Then
	$g_aLocale[$g_iNbLocale][2] = GetLocaleInfo($g_aLocale[$g_iNbLocale][0])
	$g_iNbLocale += 1
EndIf

; add missing Bangla Banglesh entry
$g_aLocale[$g_iNbLocale][0] = "0x0845"
$g_aLocale[$g_iNbLocale][1] = GetLocaleName($g_aLocale[$g_iNbLocale][0])
If Not @error Then
	$g_aLocale[$g_iNbLocale][2] = GetLocaleInfo($g_aLocale[$g_iNbLocale][0])
	$g_iNbLocale += 1
EndIf

; add missing Tamazight Morocco entry
$g_aLocale[$g_iNbLocale][0] = "0x105F"
$g_aLocale[$g_iNbLocale][1] = GetLocaleName($g_aLocale[$g_iNbLocale][0])
If Not @error Then
	$g_aLocale[$g_iNbLocale][2] = GetLocaleInfo($g_aLocale[$g_iNbLocale][0])
	$g_iNbLocale += 1
EndIf

; add missing Splitting of Serbia and Montenegro entries
$g_aLocale[$g_iNbLocale][0] = "0x241A"
$g_aLocale[$g_iNbLocale][1] = GetLocaleName($g_aLocale[$g_iNbLocale][0])
If Not @error Then
	$g_aLocale[$g_iNbLocale][2] = GetLocaleInfo($g_aLocale[$g_iNbLocale][0])
	$g_iNbLocale += 1
EndIf
$g_aLocale[$g_iNbLocale][0] = "0x281A"
$g_aLocale[$g_iNbLocale][1] = GetLocaleName($g_aLocale[$g_iNbLocale][0])
If Not @error Then
	$g_aLocale[$g_iNbLocale][2] = GetLocaleInfo($g_aLocale[$g_iNbLocale][0])
	$g_iNbLocale += 1
EndIf
$g_aLocale[$g_iNbLocale][0] = "0x2C1A"
$g_aLocale[$g_iNbLocale][1] = GetLocaleName($g_aLocale[$g_iNbLocale][0])
If Not @error Then
	$g_aLocale[$g_iNbLocale][2] = GetLocaleInfo($g_aLocale[$g_iNbLocale][0])
	$g_iNbLocale += 1
EndIf
$g_aLocale[$g_iNbLocale][0] = "0x301A"
$g_aLocale[$g_iNbLocale][1] = GetLocaleName($g_aLocale[$g_iNbLocale][0])
If Not @error Then
	$g_aLocale[$g_iNbLocale][2] = GetLocaleInfo($g_aLocale[$g_iNbLocale][0])
	$g_iNbLocale += 1
EndIf

ReDim $g_aLocale[$g_iNbLocale][3]
_ArraySort($g_aLocale)

WriteHtm()

FileClose($hList)

_OutputProgressWrite("Complete." & @CRLF)

Exit

Func OnQuit()
	_OutputWaitClosed() ; ### when run in standalone
EndFunc   ;==>OnQuit

Func WriteHtm()
	putHeader()

	; put entries
	For $i = 0 To $g_iNbLocale - 1
		If $g_aLocale[$i][0] = 0x0409 Then
			put('    <tr class="yellowbold">')
		Else
			put('    <tr>')
		EndIf
		put('      <td>' & StringTrimLeft($g_aLocale[$i][0], 2) & '</td>')
		put('      <td>' & Int($g_aLocale[$i][0]) & '</td>')
		put('      <td>' & $g_aLocale[$i][1] & '</td>')
		put('      <td>' & $g_aLocale[$i][2] & '</td>')
	Next

	putFooter()
EndFunc   ;==>WriteHtm

Func putHeader()
	put('<!DOCTYPE html>')
	put('<html>')
	put('<head>')
	put('  <title>OS Language Values-Codes</title>')
	put('  <meta charset="utf-8">')
	put('  <link href="../css/default.css" rel="stylesheet">')
	put('</head>')
	put('')
	put('<body>')
	put('<h1>@KBLayout, @MUILang, @OSLang values/codes</h1>')
	put('  <p>Possible return values (strings) of <a href="../macros/SystemInfo.htm#@KBLayout">@KBLayout</a>, <a href="../macros/SystemInfo.htm#@MUILang">@MUILang</a>, <a href="../macros/SystemInfo.htm#@OSLang">@OSLang</a><br>')
	put('  <br>')
	put('  List was generated from <a href="' & $g_sMSDNhyperlink & ' " class="ext">"' & $g_sMSDNpage & '"</a> in MSDN.<br>')
	put('  <br')
	put('  <strong>Note:</strong> Codes that contain letters could possible have the letters in uppercase.<br>')
	put('  <br></p>')
	put(' <table>')
	put('   <tr>')
	put('      <th style="width:5%">Hex</th>')
	put('      <th style="width:5%">Dec</th>')
	put('      <th style="width:10%">Country code</th>')
	put('      <th>Meaning</th>')
	put('    </tr>')
EndFunc   ;==>putHeader

Func putFooter()
	put('  </table>')
	Local $sKEYWORDS_OUT_FILE = IniRead("txt2htm.ini", "Output", "keywords", "ERR") & "@OSLang.htm"
	If FileExists($sKEYWORDS_OUT_FILE) Then
		put('')
		Local $hIn = FileOpen($sKEYWORDS_OUT_FILE)
		Local $sLine
		Do
			$sLine = FileReadLine($hIn)
		Until StringInStr($sLine, "Example</h2>")

		Do
			put($sLine)
			$sLine = FileReadLine($hIn)
		Until @error
	EndIf
EndFunc   ;==>putFooter

Func GetHeader($sLine)
	Local $asCol = StringSplit($sLine, ";")

	For $i = 1 To $asCol[0]
		If $asCol[$i] = "Locale identifier" Then
			$g_iLocale = $i
		ElseIf $asCol[$i] = "Primary language" Then
			$g_iLanguage = $i
		ElseIf $asCol[$i] = "Sublanguage" Then
			$g_iCountry = $i
		EndIf
	Next

	Return
EndFunc   ;==>GetHeader

Func GetLocale($sLine)
	$sLine = StringReplace($sLine, '; see note', '! see note')
	$sLine = StringReplace($sLine, '"', '')

	Local $asCell = StringSplit($sLine, ";")

	$g_aLocale[$g_iNbLocale][0] = StringReplace($asCell[$g_iLocale], "0c", "0C")
	If $asCell[$g_iLanguage] <> "" Then
		$g_sLanguage = StringRegExpReplace($asCell[$g_iLanguage], ' \(([^\)]*?)\)', '')
		If StringInStr($g_sLanguage, "!") Then
			$g_sLanguage = StringSplit($g_sLanguage, "!")
			$g_sLanguage = $g_sLanguage[1]
		EndIf
	EndIf

	$g_aLocale[$g_iNbLocale][1] = GetLocaleName($asCell[$g_iLocale])
	If $g_aLocale[$g_iNbLocale][1] <> "" And $g_aLocale[$g_iNbLocale][0] <> $g_iPrevLocale Then
		$g_iPrevLocale = $g_aLocale[$g_iNbLocale][0]

		Local $sCountry = StringRegExpReplace($asCell[$g_iCountry], ' \(([^\)]*?)\)', '')
		Local $sSubLanguage = ""
		If StringInStr($sCountry, "!") Then
			$sCountry = StringSplit($sCountry, "!")
			$sCountry = $sCountry[1]
		EndIf
		If StringInStr($sCountry, ",") Then
			$sCountry = StringSplit($sCountry, ",")
			$sCountry[2] = StringStripWS($sCountry[2], 3)
			If $sCountry[2] = "Norway" Or $sCountry[2] = "Finland" Or $sCountry[2] = "Sweden" Then
				$sCountry[0] = $sCountry[2]
				$sCountry[2] = $sCountry[1]
				$sCountry[1] = $sCountry[0]
			ElseIf $sCountry[2] = "Canadian Syllabics" Then
				$sCountry[2] = "Canadian_Syllabics"
			ElseIf $sCountry[0] = 3 Then
				$sCountry[2] = StringStripWS($sCountry[3], 3)
			EndIf
			If StringInStr($sCountry[2], " ") Or $sCountry[2] = "PRC" Or $sCountry[2] = "FYROM" Then $sCountry[2] = ""
			If $sCountry[2] <> "" Then $sSubLanguage = " (" & $sCountry[2] & ")"
			$sCountry = $sCountry[1]
		EndIf
		If $sCountry = "PRC" Then $sCountry = "China"
		$g_aLocale[$g_iNbLocale][2] = $g_sLanguage & $sSubLanguage & ' - ' & $sCountry

		$g_iNbLocale += 1
	EndIf
	Return
EndFunc   ;==>GetLocale

Func GetLocaleName($iLCID)
	Local $sLocalName = LCIDToLocaleName($iLCID)
	Local $iReverseLCID = LocaleNameToLCID($sLocalName)
	If $iLCID = $iReverseLCID Then Return $sLocalName

	; special case for Chinese not supported by LCIDToLocaleName
	If $iLCID = 0x0004 Then Return "zh-CHS"
	If $iLCID = 0x7C04 Then Return "zh-CHT"

	Return SetError(1, 0, "")
EndFunc   ;==>GetLocaleName

Func LCIDToLocaleName($iLCID)
	Local $aRet = DllCall("kernel32.dll", "int", "LCIDToLocaleName", "int", $iLCID, "wstr", "", "int", 85, "dword", 0)
	Return $aRet[2]
EndFunc   ;==>LCIDToLocaleName

Func LocaleNameToLCID($sStr)
	Local $aRet = DllCall("kernel32.dll", "int", "LocaleNameToLCID", "wstr", $sStr, "dword", 0)
	Return $aRet[0]
EndFunc   ;==>LocaleNameToLCID

Func GetLocaleInfo($iLCID)
	Local $iLanguageIndex = 114
	If StringInStr("WIN_2008R2 WIN_2008 WIN_VISTA WIN_2003 WIN_XP WIN_XPe", @OSVersion) Then $iLanguageIndex = 2

	Local $aTemp = DllCall('kernel32.dll', 'int', 'GetLocaleInfoW', 'ulong', $iLCID, 'dword', $iLanguageIndex, 'wstr', '', 'int', 2048)
	$aTemp[3] = StringReplace($aTemp[3], ")", "")
	If StringInStr($aTemp[3], ",") Then
		$aTemp[3] = StringReplace($aTemp[3], ",", ") -")
	Else
		$aTemp[3] = StringReplace($aTemp[3], "(", "- ")
	EndIf

	Return $aTemp[3]
EndFunc   ;==>GetLocaleInfo
; ------------------------------------------------------------------------------
; Write a new line to the output file
; ------------------------------------------------------------------------------
Func put($sLineToWrite)
	FileWriteLine($g_hOut, $sLineToWrite)
EndFunc   ;==>put
